@@ -1,6 +1,9 @@
-#$Id: Prereq.pm,v 1.21 2004/02/20 10:27:38 comdog Exp $
 package Test::Prereq;
 use strict;
+use utf8;
+
+use warnings;
+no warnings;
 
 =head1 NAME
 
@@ -24,63 +27,77 @@ Test::Prereq - check if Makefile.PL has the right pre-requisites
 	prereq_ok();
 
 	# or from the command line for a one-off check
-	perl -MTest::More=tests,1 -MTest::Prereq -eprereq_ok
+	perl -MTest::Prereq -eprereq_ok
+
+    #The prerequisites test take quite some time so the following construct is 
+    #recommended for non-author testers
+	use Test::More;
+	eval "use Test::Prereq::Build";
+
+	my $msg;
+	if ($@) {
+	    $msg = 'Test::Prereq::Build required to test dependencies';
+	} elsif (not $ENV{TEST_AUTHOR}) {
+	    $msg = 'Author test.  Set $ENV{TEST_AUTHOR} to a true value to run.';
+	}
+	plan skip_all => $msg if $msg;
+	prereq_ok();
 
 =head1 DESCRIPTION
 
-THIS IS ALPHA SOFTWARE.  IT HAS SOME PROBLEMS.
+The C<prereq_ok()> function examines the modules it finds in
+F<blib/lib/>, F<blib/script>, and the test files it finds in F<t/>
+(and F<test.pl>). It figures out which modules they use, skips the
+modules that are in the Perl core, and compares the remaining list of
+modules to those in the C<PREREQ_PM> section of F<Makefile.PL>.
 
-The prereq_ok() function examines the modules it finds in blib/lib/,
-blib/script, and the test files it finds in t/ (and test.pl). It
-figures out which modules they use, skips the modules that are in the
-Perl core, and compares the remaining list of modules to those in the
-PREREQ_PM section of Makefile.PL.
-
-If you use Module::Build instead, see L<Test::Prereq::Build> instead.
+If you use C<Module::Build> instead, see L<Test::Prereq::Build>
+instead.
 
 =head2 Modules Test::Prereq can't find
 
-Module::Info only tells Test::Prereq which modules you used, not which
-distribution they came in.  This can be a problem for things in
+C<Module::Info> only tells C<Test::Prereq> which modules you used, not
+which distribution they came in. This can be a problem for things in
 packages like libnet, libwww, Tk, and so on. At the moment
-Test::Prereq asks CPAN.pm to expand anything in PREREQ_PM to see if
-one of the distributions you explicity list contains the module you
-actually used.  This might fail in some cases.  Please send me
-anything that does not do what you think it should.
+C<Test::Prereq> asks CPAN.pm to expand anything in C<PREREQ_PM> to see
+if one of the distributions you explicity list contains the module you
+actually used. This might fail in some cases. Please send me anything
+that does not do what you think it should.
 
-Test::Prereq only asks CPAN.pm for help if it needs it, since CPAN.pm
-can be slow if it has to fetch things from the network. Once it
-fetches the right things, it should be much faster.
+C<Test::Prereq> only asks CPAN.pm for help if it needs it, since
+CPAN.pm can be slow if it has to fetch things from the network. Once
+it fetches the right things, it should be much faster.
 
 =head2 Problem with Module::Info
 
-Module::Info appears to do something wierd if a file it analyzes
-does not use (or require) any modules.  You may get a message like
+C<Module::Info> appears to do something weird if a file it analyzes
+does not use (or require) any modules. You may get a message like
 
-  Can't locate object method "name" via package "B::NULL" at
-  /usr/perl5.8.0/lib/site_perl/5.8.0/B/Module/Info.pm line 176.
+Can't locate object method "name" via package "B::NULL" at
+/usr/perl5.8.0/lib/site_perl/5.8.0/B/Module/Info.pm line 176.
 
-Also, if a file cannot compile, Module::Info dumps a lot of text
-to the terminal.  You probably want to bail out of testing if the
-files do not compile, though.
+Also, if a file cannot compile, C<Module::Info> dumps a lot of text to
+the terminal. You probably want to bail out of testing if the files do
+not compile, though.
 
 =head2 Problem with CPANPLUS
 
-CPANPLUS apparently does some wierd things, and since it is still
-young and not part of the Standard Library, Test::Prereq's tests do
-not do the right thing under it (for some reason).  Test::Prereq
-cheats by ignoring CPANPLUS completely in the tests---at least until
-someone has a better solution.  If you do not like that, you can set
-$EXCLUDE_CPANPLUS to a false value.
+C<CPANPLUS> apparently does some weird things, and since it is still
+young and not part of the Standard Library, C<Test::Prereq>'s tests do
+not do the right thing under it (for some reason). C<Test::Prereq>
+cheats by ignoring C<CPANPLUS> completely in the tests---at least
+until someone has a better solution. If you do not like that, you can
+set C<$EXCLUDE_CPANPLUS> to a false value.
 
 You should be able to do a 'make test' manually to make everything
 work, though.
 
 =head2 Warning about redefining ExtUtils::MakeMaker::WriteMakefile
 
-Test::Prereq has its own version of ExtUtils::MakeMaker::WriteMakefile
-so it can run the Makefile.PL and get the argument list of that
-function.  You may see warnings about this.
+C<Test::Prereq> has its own version of
+C<ExtUtils::MakeMaker::WriteMakefile> so it can run the F<Makefile.PL>
+and get the argument list of that function.  You may see warnings
+about this.
 
 =cut
 
@@ -88,17 +105,17 @@ use base qw(Exporter);
 use vars qw($VERSION $EXCLUDE_CPANPLUS @EXPORT @prereqs);
 
 
-$VERSION = '0.23';
+$VERSION = '1.037_02';
+
 @EXPORT = qw( prereq_ok );
 
 use Carp qw(carp);
-use CPAN;
-use Exporter;
 use ExtUtils::MakeMaker;
 use File::Find;
 use Module::CoreList;
 use Module::Info;
 use Test::Builder;
+use Test::More;
 
 my $Test = Test::Builder->new;
 
@@ -114,62 +131,80 @@ no warnings;
 	my %hash = @_;
 
 	my $name = $hash{NAME};
-	my $hash = $hash{PREREQ_PM};
+	my %prereqs =
+		map { defined $_ ? %$_ : () }
+		@hash{qw(PREREQ_PM BUILD_REQUIRES CONFIGURE_REQUIRES)};
 
 	$Namespace = $name;
-	@Test::Prereq::prereqs   = sort keys %$hash;
+	@Test::Prereq::prereqs   = sort keys %prereqs;
+	
+	1;
 	}
 }
 
+#unless( caller ) { prereq_ok() }
+
 =head1 FUNCTIONS
 
 =over 4
 
 =item prereq_ok( [ VERSION, [ NAME [, SKIP_ARRAY] ] ] )
 
-If you don't specify a version, prereq_ok assumes you want
-to compare the list of prerequisite modules to version
-5.008001.
+Tests F<Makefile.PL> to ensure all non-core module dependencies are in
+C<PREREQ_PM>. If you haven't set a testing plan already,
+C<prereq_ok()> creates a plan of one test.
 
-Valid versions come from Module::CoreList (which uses $[).
+If you don't specify a version, C<prereq_ok> assumes you want to compare
+the list of prerequisite modules to the version of perl running the
+test.
+
+Valid versions come from C<Module::CoreList> (which uses C<$]>).
 
 	#!/usr/bin/perl
 	use Module::CoreList;
 	print map "$_\n", sort keys %Module::CoreList::version;
 
-
-	5.00307
-	5.004
-	5.00405
-	5.005
-	5.00503
-	5.006
-	5.006001
-	5.006002
-	5.007003
-	5.008
-	5.008001
-	5.008002
-	5.009
-
-prereq_ok attempts to remove modules found in blib and
-libraries found in t from the reported prerequisites.
+C<prereq_ok> attempts to remove modules found in F<lib/> and
+libraries found in F<t/> from the reported prerequisites.
 
 The optional third argument is an array reference to a list
-of names that prereq_ok should ignore. You might want to use
-this if your tests do funny things with require.
+of names that C<prereq_ok> should ignore. You might want to use
+this if your tests do funny things with C<require>.
+
+Versions prior to 1.038 would use CPAN.pm to virtually include
+prerequisites in distributions that you declared explicitly. This isn't
+really a good idea. Some modules have moved to different distributions,
+so you should just specify all the modules that you use instead of relying
+on a particular distribution to provide them. Not only that, expanding
+distributions with CPAN.pm takes forever.
+
+If you want the old behavior, set the C<TEST_PREREQ_EXPAND_WITH_CPAN> 
+environment variable to a true value.
+
+
 
 =cut
 
-my $default_version = '5.008001';
-my $version         = '5.008001';
+my $default_version = $];
+my $version         = $];
 
 sub prereq_ok
 	{
-	$Test->plan( tests => 1 );
+	$Test->plan( tests => 1 ) unless $Test->has_plan;
 	__PACKAGE__->_prereq_check( @_ );
 	}
 
+sub import 
+	{
+    my $self   = shift;
+    my $caller = caller;
+    no strict 'refs';
+    *{$caller.'::prereq_ok'}       = \&prereq_ok;
+
+    $Test->exported_to($caller);
+    $Test->plan(@_);
+	}
+
 sub _prereq_check
 	{
 	my $class   = shift;
@@ -181,7 +216,7 @@ sub _prereq_check
 	$version = $default_version unless
 		exists $Module::CoreList::version{$version};
 
-	unless( UNIVERSAL::isa( $skip, 'ARRAY' ) )
+	unless( ref $skip eq ref [] )
 		{
 		carp( 'Third parameter to prereq_ok must be an array reference!' );
 		return;
@@ -235,11 +270,11 @@ sub _prereq_check
 	# if anything is left, look for modules in the distributions
 	# in PREREQ_PM.  this is slow, so we should only do it if
 	# we might need it.
-	if( keys %$loaded )
+	if( keys %$loaded and $class->_should_i_expand_prereqs )
 		{
 		my $modules = $class->_get_from_prereqs( $prereqs );
 
-		foreach my $module ( @$skip )
+		foreach my $module ( @$modules )
 			{
 			delete $loaded->{$module};
 			}
@@ -256,7 +291,7 @@ sub _prereq_check
 
 	if( keys %$loaded ) # stuff left in %loaded, oops!
 		{
-		$class->_not_ok( "Found some modules that didn't show up in PREREQ_PM\n",
+		$class->_not_ok( "Found some modules that didn't show up in PREREQ_PM or *_REQUIRES\n",
 			map { "\t$_\n" } sort keys %$loaded );
 		}
 	else
@@ -285,11 +320,11 @@ sub _get_prereqs
 	delete $INC{$file};  # make sure we load it again
 
 	{
-	no warnings;
+	local $^W = 0;
 	
 	unless( do "./$file" )
 		{
-		print STDERR "_get_prereqs: Error loading $file: $!";
+		print STDERR "_get_prereqs: Error loading $file: $@\n";
 		return;
 		}
 	delete $INC{$file};  # pretend we were never here
@@ -301,13 +336,18 @@ sub _get_prereqs
 	}
 
 # expand prereqs and see what we get
+
+sub _should_i_expand_prereqs { !! $ENV{TEST_PREREQ_EXPAND_WITH_CPAN} }
+
 sub _get_from_prereqs
 	{
 	my $class   = shift;
 	my $modules = shift;
 
 	my @dist_modules = ();
-
+	return [] unless $class->_should_i_expand_prereqs;
+	
+	require CPAN;
 	foreach my $module ( @$modules )
 		{
 		my $mod      = CPAN::Shell->expand( "Module", $module );
@@ -334,11 +374,11 @@ sub _get_loaded_modules
 
 	my( @libs, @t, @scripts );
 	
-	find( sub { push @libs,    $File::Find::name if m/\.pm$/ }, 'blib/lib' )
+	File::Find::find( sub { push @libs,    $File::Find::name if m/\.pm$/ }, 'blib/lib' )
 		if -e 'blib/lib';
-	find( sub { push @t,       $File::Find::name if m/\.t$/  }, 't' )
+	File::Find::find( sub { push @t,       $File::Find::name if m/\.t$/  }, 't' )
 		if -e 't';
-	find( sub { push @scripts, $File::Find::name if -f $_    }, 'blib/script' )
+	File::Find::find( sub { push @scripts, $File::Find::name if -f $_    }, 'blib/script' )
 		if -e 'blib/script';
 	
 	my @found = ();
@@ -358,7 +398,7 @@ sub _get_test_libraries
 
 	my @found = ();
 	
-	find( sub { push @found, $File::Find::name if m/\.p(l|m)$/ }, 't' );
+	File::Find::find( sub { push @found, $File::Find::name if m/\.p(l|m)$/ }, 't' );
 
 	my @files =
 		map {
@@ -384,7 +424,7 @@ sub _get_dist_modules
 
 	my @found = ();
 	
-	find( sub { push @found, $File::Find::name if m/\.pm$/ }, $_[0] );
+	File::Find::find( sub { push @found, $File::Find::name if m/\.pm$/ }, $_[0] );
 		
 	my @files =
 		map {
@@ -404,8 +444,9 @@ sub _get_from_file
 	my( $class, $file ) = @_;
 
 	my $module  = Module::Info->new_from_file( $file );
+	$module->die_on_compilation_error(1);
 
-	my @used    = $module->modules_used;
+	my @used    = eval{ $module->modules_used };
 
 	my @modules =
 		sort
@@ -423,36 +464,29 @@ sub _get_from_file
 
 * set up a couple fake module distributions to test
 
-* warn about things that show up in PREREQ_PM unnecessarily
-
-* get rid of File::Finder::Rule
+* warn about things that show up in C<PREREQ_PM> unnecessarily
 
 =head1 SOURCE AVAILABILITY
 
-This source is part of a SourceForge project which always has the
-latest sources in CVS, as well as all of the previous releases.
+This source is in Github:
 
-	https://sourceforge.net/projects/brian-d-foy/
-
-If, for some reason, I disappear from the world, one of the other
-members of the project can shepherd this module appropriately.
+	http://github.com/briandfoy/test-prereq
 
 =head1 CONTRIBUTORS
 
 Many thanks to:
 
-Andy Lester, Slavin Rezic, Randal Schwartz, Iain Truskett
+Andy Lester, Slavin Rezić, Randal Schwartz, Iain Truskett, Dylan Martin
 
 =head1 AUTHOR
 
-brian d foy, E<lt>bdfoy@cpan.orgE<gt>
+brian d foy, C<< <bdfoy@cpan.org> >>
 
-=head1 COPYRIGHT
+=head1 COPYRIGHT and LICENSE
 
-Copyright 2002, brian d foy, All Rights Reserved.
+Copyright 2002-2010, brian d foy, All rights reserved
 
-You may use, modify, and distribute this package under the
-same terms as Perl itself.
+This software is available under the same terms as perl.
 
 =cut
 
